-- This file contains NetApp-specific extensions
local config = require('config')
local hook = require('hook')

local function print_env_err(msg)
	print(" ")
	if msg ~= nil then
		print(msg)
	end
	print("******************************************* ")
	print("Unable to read environment from boot media! ")
	print("Please re-create your VM environment. ")
	print("******************************************* ")
end

local function include_env_failed(msg)
	loader.setenv("bootarg.envfailed", 1)
	loader.setenv("console", "vidconsole")
	print_env_err(msg)
	loader.setenv("console", "comconsole")
	print_env_err(msg)
	loader.setenv("console", "vidconsole")
end

local function process_env_file(filename)
	print("Processing " .. filename)
	local f = io.open(filename)
	if f == nil then
		return nil, "Cannot open " .. filename
	end

	local text = io.read(f)
	io.close(f)

	if text == nil then
		return nil, "Failed to read " .. filename
	end

	local kv_expression = "^([^ ]+) (.+)"
	-- Process the text, line by line
	for line in text:gmatch("([^\n]+)") do
		if line:match("^setenv ") then
			-- Process setenv lines; we'll skip past the literal 'setenv ' and
			-- then peel off a name and value.
			local kv_line = line:sub(8)
			local key, value = kv_line:match(kv_expression)
			-- Parse error; something went wrong, we should have had two capture
			-- groups from the line expression.  The line expression is
			-- intentionally super-lax and doesn't do any validation on either,
			-- just grabs the first sequence of characters that don't include
			-- a space and calls that the key; everything after the space is
			-- the value.
			if key == nil or value == nil then
				return nil, "Failed to parse, line: " .. line
			end
			if key:match("=") then
				return nil, "Key cannot contain =, line: " .. line
			end
			-- Strip off any leading/trailing space
			value = value:gsub("^ +", ""):gsub(" +$", "")
			loader.perform("set " .. key .."=" .. value)
		end
	end

	return true
end

local function include_env_files()
	local filename = '/env/env'
	local loaddevs = {}

	-- /env/env may switch currdev, and we must then process
	-- /env/env on the new currdev to get the complete picture.
	--
	-- In testing, the boot partition had only a limited env.
::reload::
	local startdev = loader.getenv('currdev')
	if loaddevs[startdev] ~= nil then
		return
	end

	loaddevs[startdev] = true

	-- Check existence of /env/env
	if lfs.attributes(filename) == nil then
		print("/env/env is not found. Use /env_bak/env instead")

		filename = '/env_bak/env'
		-- Now /env_bak/env
		if lfs.attributes(filename) == nil then
			print("/env_bak/env not found either.")
			include_env_failed()
			return
		end
	end

	-- Now we have a solid idea of what to read stashed in filename; do it.
	local ok, msg = process_env_file(filename)
	if not ok then
		include_env_failed(msg)
		return
	end

	if loader.getenv('currdev') ~= startdev then
		goto reload
	end
end

-- Stub out loadkernel; NetApp has a different mechanism that loads the kernel,
-- so we just need to pretend that it succeeded here.
function config.loadKernel()
	return true
end

hook.register("config.reloaded", include_env_files)

-- We have a config.loaded hook as well, but the local module is included too
-- late to use it.  At this point, config is already loaded, so go ahead and
-- try to process the env files.  The above registered hook will ensure we
-- re-read them if we, for any reason, reload config files.  If the local env
-- overwrites loader.conf, for instance, then this is important.
include_env_files()
